#!/usr/bin/perl

use IMDB::Film;

my $DEBUG = 1;
$| = 1;

my $blacklist_file = "./blacklist.txt";
my %hash;
my %blacklist;

# load blacklist
open IN, "<$blacklist_file";
while (<IN>){
	chomp;
	$blacklist{$_}++;
}
close IN;

if (-e $ARGV[0]){
	# the argument passed is a file
	# in this use, we suppose that the file contains a list of filenames
	# (this is used for testing the tool)
	print "The parameter you passed has been recognized as a file. \nParsing ".$ARGV[0]." ...\n";
	
	open IN, "<".$ARGV[0];
	while (<IN>){
	        print;
	        my $cleanTitle = cleanTitle($_, \%blacklist);
		printInfoForTitle($cleanTitle);
		sleep 4;
	}
}else{
	# 1) Clean the title passed as the first argument to the script
	my $cleanTitle = cleanTitle($ARGV[0], \%blacklist);
	
	# 2) Search IMDB and print information related to $cleanTitle
	printInfoForTitle($cleanTitle);
}



exit;

# printInfoForTitle is used to show you how you can easily get structured info
# about a movie once you have a clean title. Some test methods are called (see
# IMDB::Film documentation for more), then a search for alternative titles
# (biased towards Italian ones) is issued

sub printInfoForTitle{
	my $title = shift;

	print "\t[i] Searching for $title...\n" if $DEBUG;
	my $imdb = new IMDB::Film(crit => "$title", debug => $DEBUG==2);
	
	if($imdb->status) {
		# print full movie title (as identified by IMDB)
	        print "\t==> Title: ".$imdb->title()." - ";
		
		# print movie year
	        print "Year: ".$imdb->year()." - ";

		# compose list of directors as a string and print it
		my @directors = @{ $imdb->directors() };
		my @dnames;
		foreach $director (@directors){
			push @dnames, $$director{name};
		}
		print "Directors: " . join (",", @dnames)."\n";
	
		# get alternative title and print it	
		print "\t    Alternative title: ".getAlternativeTitle($imdb)."\n\n";
	
	} else {
		print "Something wrong: ".$imdb->error;
	}
}

# getAlternativeTitle relies on a modification of Film.pm (IMDB::Film) that
# returns a hash instead of an array. See the following URL:
#
# to read an explaination and get the diff files

sub getAlternativeTitle{
	my $imdb = shift;
	my $altTitle = "";
	my $aka = $imdb->also_known_as();
	foreach $key (keys %$aka){
		# NOTE: currently the default is to return the Italian title,
		# otherwise rollback to the first occurrence of International
		# or English. Change below here if you want to customize it!
		if ($key =~ /^Ital/){
			$altTitle = $$aka{$key};
			last;
		}elsif ($key =~ /^(International|English)/){
			$altTitle = $$aka{$key};
		}
	}
	return $altTitle;
}

# The idea of cleanTitle is to get the title as an input string, split it 
# into an array of tokens, and then eliminate all those tokens that are
# supposedly not part of the title. This can be done with different
# heuristics...

sub cleanTitle{
	my ($dirtyTitle, $blacklist) = @_;
	my @cleanTitleArray;
	
	# HEURISTIC #1: everything which is not alphanumeric is a separator;
	# HEURISTIC #2: if an extracted word belongs to the blacklist then ignore it;
	while ($dirtyTitle =~ /([a-zA-Z0-9]+)/g){
		my $word = lc($1);    # blacklist is lowercase
		if (!defined $$blacklist{$word}){
			push @cleanTitleArray, $word;
		}
	}
	
	# HEURISTIC #3: often movies have a date (year) after the title, remove
	# that (if it is not the title of the movie itself!);
	my $lastWord = pop(@cleanTitleArray);
	my $arraySize = @cleanTitleArray;
	if ($lastWord !~ /(19\d\d)|(20\d\d)/ || !$arraySize){
		push @cleanTitleArray, $lastWord;
	} 

	return join (" ", @cleanTitleArray);
}

